@@ -94,7 +94,7 @@ void CControlSocket::LogTransferResultMessage(int nErrorCode, CFileTransferOpDat
 
 	CTransferStatus const status = engine_.transfer_status_.Get(tmp);
 	if (!status.empty() && (nErrorCode == FZ_REPLY_OK || status.madeProgress)) {
-		int elapsed = wxTimeSpan(wxDateTime::UNow() - status.started).GetSeconds().GetLo();
+		int elapsed = static_cast<int>((CDateTime::Now() - status.started).get_seconds());
 		if (elapsed <= 0)
 			elapsed = 1;
 		wxString time = wxString::Format(
@@ -548,20 +548,20 @@ void CControlSocket::OnTimer(timer_id)
 
 	int const timeout = engine_.GetOptions().GetOptionVal(OPTION_TIMEOUT);
 	if (timeout > 0) {
-		int64_t elapsed = CMonotonicClock::now() - m_lastActivity;
+		duration elapsed = CMonotonicClock::now() - m_lastActivity;
 
 		if ((!m_pCurOpData || !m_pCurOpData->waitForAsyncRequest) && !IsWaitingForLock()) {
-			if (elapsed > static_cast<int64_t>(timeout) * 1000) {
+			if (elapsed > duration::from_seconds(timeout)) {
 				LogMessage(MessageType::Error, wxPLURAL("Connection timed out after %d second of inactivity", "Connection timed out after %d seconds of inactivity", timeout), timeout);
 				DoClose(FZ_REPLY_TIMEOUT);
 				return;
 			}
 		}
 		else {
-			elapsed = 0;
+			elapsed = duration();
 		}
 
-		m_timer = AddTimer(timeout * 1000 - elapsed, true);
+		m_timer = AddTimer(duration::from_milliseconds(timeout * 1000) - elapsed, true);
 	}
 }
 
@@ -582,7 +582,7 @@ void CControlSocket::SetWait(bool wait)
 		if (!timeout)
 			return;
 
-		m_timer = AddTimer(timeout * 1000 + 100, true); // Add a bit of slack
+		m_timer = AddTimer(duration::from_milliseconds(timeout * 1000 + 100), true); // Add a bit of slack
 	}
 	else {
 		StopTimer(m_timer);
@@ -809,16 +809,16 @@ void CControlSocket::InvalidateCurrentWorkingDir(const CServerPath& path)
 	}
 }
 
-wxTimeSpan CControlSocket::GetTimezoneOffset()
+duration CControlSocket::GetTimezoneOffset() const
 {
-	if (!m_pCurrentServer)
-		return wxTimeSpan();
-
-	int seconds = 0;
-	if (CServerCapabilities::GetCapability(*m_pCurrentServer, timezone_offset, &seconds) != yes)
-		return wxTimeSpan();
-
-	return wxTimeSpan(0, 0, seconds);
+	duration ret;
+	if (m_pCurrentServer) {
+		int seconds = 0;
+		if (CServerCapabilities::GetCapability(*m_pCurrentServer, timezone_offset, &seconds) == yes) {
+			ret = duration::from_seconds(seconds);
+		}
+	}
+	return ret;
 }
 
 void CControlSocket::SendAsyncRequest(CAsyncRequestNotification* pNotification)
