@@ -228,7 +228,7 @@ int CFileZillaEnginePrivate::ResetOperation(int nErrorCode)
 							delay = 1;
 						m_pLogging->LogMessage(MessageType::Status, _("Waiting to retry..."));
 						StopTimer(m_retryTimer);
-						m_retryTimer = AddTimer(delay, true);
+						m_retryTimer = AddTimer(duration::from_milliseconds(delay), true);
 						return FZ_REPLY_WOULDBLOCK;
 					}
 				}
@@ -296,11 +296,9 @@ int CFileZillaEnginePrivate::Connect(const CConnectCommand &command)
 
 int CFileZillaEnginePrivate::Disconnect(CDisconnectCommand const&)
 {
-	if (!IsConnected())
-		return FZ_REPLY_OK;
-
-	int res = m_pControlSocket->Disconnect();
-	if (res == FZ_REPLY_OK) {
+	int res = FZ_REPLY_OK;
+	if (m_pControlSocket) {
+		res = m_pControlSocket->Disconnect();
 		m_pControlSocket.reset();
 	}
 
@@ -329,7 +327,7 @@ int CFileZillaEnginePrivate::List(const CListCommand &command)
 					else {
 						if (!avoid) {
 							m_lastListDir = pListing->path;
-							m_lastListTime = CDateTime::Now();
+							m_lastListTime = CMonotonicClock::now();
 							CDirectoryListingNotification *pNotification = new CDirectoryListingNotification(pListing->path);
 							AddNotification(pNotification);
 						}
@@ -405,13 +403,13 @@ void CFileZillaEnginePrivate::SendDirectoryListingNotification(const CServerPath
 
 	if (failed) {
 		AddNotification(new CDirectoryListingNotification(path, false, true));
-		m_lastListTime = CMonotonicTime::Now();
+		m_lastListTime = CMonotonicClock::now();
 
 		// On failed messages, we don't notify other engines
 		return;
 	}
 
-	CMonotonicTime changeTime;
+	CMonotonicClock changeTime;
 	if (!directory_cache_.GetChangeTime(changeTime, *pOwnServer, path))
 		return;
 
@@ -424,17 +422,18 @@ void CFileZillaEnginePrivate::SendDirectoryListingNotification(const CServerPath
 	// Iterate over the other engine, send notification if last listing
 	// directory is the same
 	for (auto & engine : m_engineList) {
-		if (!engine->m_pControlSocket || engine->m_pControlSocket == m_pControlSocket)
+		if (!engine->m_pControlSocket || engine->m_pControlSocket == m_pControlSocket) {
 			continue;
+		}
 
-		const CServer* const pServer = engine->m_pControlSocket->GetCurrentServer();
+		CServer const* const pServer = engine->m_pControlSocket->GetCurrentServer();
 		if (!pServer || *pServer != *pOwnServer)
 			continue;
 
 		if (engine->m_lastListDir != path)
 			continue;
 
-		if (engine->m_lastListTime.GetTime().IsValid() && changeTime <= engine->m_lastListTime)
+		if (engine->m_lastListTime && changeTime <= engine->m_lastListTime)
 			continue;
 
 		engine->m_lastListTime = changeTime;
@@ -446,10 +445,9 @@ void CFileZillaEnginePrivate::RegisterFailedLoginAttempt(const CServer& server,
 {
 	scoped_lock lock(mutex_);
 	std::list<t_failedLogins>::iterator iter = m_failedLogins.begin();
-	while (iter != m_failedLogins.end())
-	{
-		const wxTimeSpan span = wxDateTime::UNow() - iter->time;
-		if (span.GetSeconds() >= m_options.GetOptionVal(OPTION_RECONNECTDELAY) ||
+	while (iter != m_failedLogins.end()) {
+		duration const span = CDateTime::Now() - iter->time;
+		if (span.get_seconds() >= m_options.GetOptionVal(OPTION_RECONNECTDELAY) ||
 			iter->server == server || (!critical && (iter->server.GetHost() == server.GetHost() && iter->server.GetPort() == server.GetPort())))
 		{
 			std::list<t_failedLogins>::iterator prev = iter;
@@ -462,7 +460,7 @@ void CFileZillaEnginePrivate::RegisterFailedLoginAttempt(const CServer& server,
 
 	t_failedLogins failure;
 	failure.server = server;
-	failure.time = wxDateTime::UNow();
+	failure.time = CDateTime::Now();
 	failure.critical = critical;
 	m_failedLogins.push_back(failure);
 }
@@ -471,20 +469,18 @@ unsigned int CFileZillaEnginePrivate::GetRemainingReconnectDelay(const CServer&
 {
 	scoped_lock lock(mutex_);
 	std::list<t_failedLogins>::iterator iter = m_failedLogins.begin();
-	while (iter != m_failedLogins.end())
-	{
-		const wxTimeSpan span = wxDateTime::UNow() - iter->time;
+	while (iter != m_failedLogins.end()) {
+		duration const span = CDateTime::Now() - iter->time;
 		const int delay = m_options.GetOptionVal(OPTION_RECONNECTDELAY);
-		if (span.GetSeconds() >= delay)
-		{
+		if (span.get_seconds() >= delay) {
 			std::list<t_failedLogins>::iterator prev = iter;
 			++iter;
 			m_failedLogins.erase(prev);
 		}
 		else if (!iter->critical && iter->server.GetHost() == server.GetHost() && iter->server.GetPort() == server.GetPort())
-			return delay * 1000 - span.GetMilliseconds().GetLo();
+			return delay * 1000 - span.get_milliseconds();
 		else if (iter->server == server)
-			return delay * 1000 - span.GetMilliseconds().GetLo();
+			return delay * 1000 - span.get_milliseconds();
 		else
 			++iter;
 	}
@@ -514,13 +510,18 @@ int CFileZillaEnginePrivate::ContinueConnect()
 {
 	scoped_lock lock(mutex_);
 
+	if (!m_pCurrentCommand || m_pCurrentCommand->GetId() != Command::connect) {
+		m_pLogging->LogMessage(MessageType::Debug_Warning, _T("CFileZillaEnginePrivate::ContinueConnect called without pending Command::connect"));
+		return ResetOperation(FZ_REPLY_INTERNALERROR);
+	}
+
 	const CConnectCommand *pConnectCommand = static_cast<CConnectCommand *>(m_pCurrentCommand.get());
 	const CServer& server = pConnectCommand->GetServer();
 	unsigned int delay = GetRemainingReconnectDelay(server);
 	if (delay) {
 		m_pLogging->LogMessage(MessageType::Status, wxPLURAL("Delaying connection for %d second due to previously failed connection attempt...", "Delaying connection for %d seconds due to previously failed connection attempt...", (delay + 999) / 1000), (delay + 999) / 1000);
 		StopTimer(m_retryTimer);
-		m_retryTimer = AddTimer(delay, true);
+		m_retryTimer = AddTimer(duration::from_milliseconds(delay), true);
 		return FZ_REPLY_WOULDBLOCK;
 	}
 
@@ -597,6 +598,9 @@ int CFileZillaEnginePrivate::CheckCommandPreconditions(CCommand const& command,
 	else if (command.GetId() != Command::connect && command.GetId() != Command::disconnect && !IsConnected()) {
 		return FZ_REPLY_NOTCONNECTED;
 	}
+	else if (command.GetId() == Command::connect && m_pControlSocket) {
+		return FZ_REPLY_ALREADYCONNECTED;
+	}
 	return FZ_REPLY_OK;
 }
 
@@ -873,7 +877,7 @@ void CTransferStatusManager::SetStartTime()
 	if (!status_)
 		return;
 
-	status_.started = wxDateTime::UNow();
+	status_.started = CDateTime::Now();
 }
 
 void CTransferStatusManager::SetMadeProgress()
