@@ -6,7 +6,7 @@
 #include "event_loop.h"
 #include "pathcache.h"
 #include "local_filesys.h"
-#include "process.h"
+#include "fzprocess.h"
 #include "proxy.h"
 #include "servercapabilities.h"
 #include "sftpcontrolsocket.h"
@@ -299,14 +299,12 @@ loopexit:
 	mutex m_sync;
 };
 
-class CSftpDeleteOpData : public COpData
+class CSftpDeleteOpData final : public COpData
 {
 public:
 	CSftpDeleteOpData()
 		: COpData(Command::del)
 	{
-		m_needSendListing = false;
-		m_deleteFailed = false;
 	}
 
 	virtual ~CSftpDeleteOpData() {}
@@ -314,14 +312,14 @@ public:
 	CServerPath path;
 	std::deque<wxString> files;
 
-	// Set to wxDateTime::UNow initially and after
+	// Set to CDateTime::Now initially and after
 	// sending an updated listing to the UI.
-	wxDateTime m_time;
+	CDateTime m_time;
 
-	bool m_needSendListing;
+	bool m_needSendListing{};
 
 	// Set to true if deletion of at least one file failed
-	bool m_deleteFailed;
+	bool m_deleteFailed{};
 };
 
 CSftpControlSocket::CSftpControlSocket(CFileZillaEnginePrivate & engine)
@@ -1007,31 +1005,23 @@ int CSftpControlSocket::ListParseResponse(bool successful, const wxString& reply
 				seconds += c - '0';
 			}
 			if (parsed) {
-				wxDateTime date = wxDateTime(seconds);
+				CDateTime date(seconds, CDateTime::seconds);
 				if (date.IsValid()) {
-					date.MakeTimezone(wxDateTime::GMT0);
 					wxASSERT(pData->directoryListing[pData->mtime_index].has_date());
-					wxDateTime listTime = pData->directoryListing[pData->mtime_index].time.Degenerate();
-					listTime += wxTimeSpan(0, -m_pCurrentServer->GetTimezoneOffset(), 0);
+					CDateTime listTime = pData->directoryListing[pData->mtime_index].time;
+					listTime -= duration::from_minutes(m_pCurrentServer->GetTimezoneOffset());
 
-					int serveroffset = (date - listTime).GetSeconds().GetLo();
-					if (!pData->directoryListing[pData->mtime_index].has_seconds())
-					{
+					int serveroffset = static_cast<int>((date - listTime).get_seconds());
+					if (!pData->directoryListing[pData->mtime_index].has_seconds()) {
 						// Round offset to full minutes
 						if (serveroffset < 0)
 							serveroffset -= 59;
 						serveroffset -= serveroffset % 60;
 					}
 
-					wxDateTime now = wxDateTime::Now();
-					wxDateTime now_utc = now.ToTimezone(wxDateTime::GMT0);
+					LogMessage(MessageType::Status, _("Timezone offset of server is %d seconds."), -serveroffset);
 
-					int localoffset = (now - now_utc).GetSeconds().GetLo();
-					int offset = serveroffset + localoffset;
-
-					LogMessage(MessageType::Status, _("Timezone offsets: Server: %d seconds. Local: %d seconds. Difference: %d seconds."), -serveroffset, localoffset, offset);
-
-					wxTimeSpan span(0, 0, offset);
+					duration span = duration::from_seconds(serveroffset);
 					const int count = pData->directoryListing.GetCount();
 					for (int i = 0; i < count; ++i) {
 						CDirentry& entry = pData->directoryListing[i];
@@ -1040,7 +1030,7 @@ int CSftpControlSocket::ListParseResponse(bool successful, const wxString& reply
 
 					// TODO: Correct cached listings
 
-					CServerCapabilities::SetCapability(*m_pCurrentServer, timezone_offset, yes, offset);
+					CServerCapabilities::SetCapability(*m_pCurrentServer, timezone_offset, yes, serveroffset);
 				}
 			}
 		}
@@ -1105,7 +1095,7 @@ int CSftpControlSocket::ListParseEntry(const wxString& entry)
 		return FZ_REPLY_INTERNALERROR;
 	}
 
-	pData->pParser->AddLine(entry.wx_str());
+	pData->pParser->AddLine(entry.c_str());
 
 	return FZ_REPLY_WOULDBLOCK;
 }
@@ -1803,7 +1793,7 @@ int CSftpControlSocket::FileTransferSend()
 
 		wxString quotedFilename = QuoteFilename(pData->remotePath.FormatFilename(pData->remoteFile, !pData->tryAbsolutePath));
 		// Y2K38
-		time_t ticks = pData->fileTime.Degenerate().GetTicks(); // Already in UTC
+		time_t ticks = pData->fileTime.GetTimeT();
 		wxString seconds = wxString::Format(_T("%d"), (int)ticks);
 		if (!SendCommand(_T("chmtime ") + seconds + _T(" ") + WildcardEscape(quotedFilename),
 			_T("chmtime ") + seconds + _T(" ") + quotedFilename))
@@ -1826,57 +1816,47 @@ int CSftpControlSocket::FileTransferParseResponse(bool successful, const wxStrin
 
 	CSftpFileTransferOpData *pData = static_cast<CSftpFileTransferOpData *>(m_pCurOpData);
 
-	if (pData->opState == filetransfer_transfer)
-	{
-		if (!successful)
-		{
+	if (pData->opState == filetransfer_transfer) {
+		if (!successful) {
 			ResetOperation(FZ_REPLY_ERROR);
 			return FZ_REPLY_ERROR;
 		}
 
-		if (engine_.GetOptions().GetOptionVal(OPTION_PRESERVE_TIMESTAMPS))
-		{
-			if (pData->download)
-			{
-				if (pData->fileTime.IsValid())
-				{
+		if (engine_.GetOptions().GetOptionVal(OPTION_PRESERVE_TIMESTAMPS)) {
+			if (pData->download) {
+				if (pData->fileTime.IsValid()) {
 					if (!CLocalFileSystem::SetModificationTime(pData->localFile, pData->fileTime))
 						LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Warning, _T("Could not set modification time"));
 				}
 			}
-			else
-			{
+			else {
 				pData->fileTime = CLocalFileSystem::GetModificationTime(pData->localFile);
-				if (pData->fileTime.IsValid())
-				{
+				if (pData->fileTime.IsValid()) {
 					pData->opState = filetransfer_chmtime;
 					return SendNextCommand();
 				}
 			}
 		}
 	}
-	else if (pData->opState == filetransfer_mtime)
-	{
-		if (successful && !reply.empty())
-		{
+	else if (pData->opState == filetransfer_mtime) {
+		if (successful && !reply.empty()) {
 			time_t seconds = 0;
 			bool parsed = true;
-			for (unsigned int i = 0; i < reply.Len(); ++i)
-			{
+			for (unsigned int i = 0; i < reply.Len(); ++i) {
 				wxChar c = reply[i];
-				if (c < '0' || c > '9')
-				{
+				if (c < '0' || c > '9') {
 					parsed = false;
 					break;
 				}
 				seconds *= 10;
 				seconds += c - '0';
 			}
-			if (parsed)
-			{
-				CDateTime fileTime = CDateTime(wxDateTime(seconds), CDateTime::seconds);
-				if (fileTime.IsValid())
+			if (parsed) {
+				CDateTime fileTime = CDateTime(seconds, CDateTime::seconds);
+				if (fileTime.IsValid()) {
 					pData->fileTime = fileTime;
+					pData->fileTime += duration::from_minutes(m_pCurrentServer->GetTimezoneOffset());
+				}
 			}
 		}
 		pData->opState = filetransfer_transfer;
@@ -1886,17 +1866,14 @@ int CSftpControlSocket::FileTransferParseResponse(bool successful, const wxStrin
 
 		return SendNextCommand();
 	}
-	else if (pData->opState == filetransfer_chmtime)
-	{
-		if (pData->download)
-		{
+	else if (pData->opState == filetransfer_chmtime) {
+		if (pData->download) {
 			LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Info, _T("  filetransfer_chmtime during download"));
 			ResetOperation(FZ_REPLY_INTERNALERROR);
 			return FZ_REPLY_ERROR;
 		}
 	}
-	else
-	{
+	else {
 		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Info, _T("  Called at improper time: opState == %d"), pData->opState);
 		ResetOperation(FZ_REPLY_INTERNALERROR);
 		return FZ_REPLY_ERROR;
@@ -2143,8 +2120,7 @@ int CSftpControlSocket::DeleteParseResponse(bool successful, const wxString&)
 {
 	LogMessage(MessageType::Debug_Verbose, _T("CSftpControlSocket::DeleteParseResponse"));
 
-	if (!m_pCurOpData)
-	{
+	if (!m_pCurOpData) {
 		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Info, _T("Empty m_pCurOpData"));
 		ResetOperation(FZ_REPLY_INTERNALERROR);
 		return FZ_REPLY_ERROR;
@@ -2154,15 +2130,13 @@ int CSftpControlSocket::DeleteParseResponse(bool successful, const wxString&)
 
 	if (!successful)
 		pData->m_deleteFailed = true;
-	else
-	{
+	else {
 		const wxString& file = pData->files.front();
 
 		engine_.GetDirectoryCache().RemoveFile(*m_pCurrentServer, pData->path, file);
 
-		wxDateTime now = wxDateTime::UNow();
-		if (now.IsValid() && pData->m_time.IsValid() && (now - pData->m_time).GetSeconds() >= 1)
-		{
+		auto const now = CDateTime::Now();
+		if (now.IsValid() && pData->m_time.IsValid() && (now - pData->m_time).get_seconds() >= 1) {
 			engine_.SendDirectoryListingNotification(pData->path, false, true, false);
 			pData->m_time = now;
 			pData->m_needSendListing = false;
@@ -2183,8 +2157,7 @@ int CSftpControlSocket::DeleteSend()
 {
 	LogMessage(MessageType::Debug_Verbose, _T("CSftpControlSocket::DeleteSend"));
 
-	if (!m_pCurOpData)
-	{
+	if (!m_pCurOpData) {
 		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Info, _T("Empty m_pCurOpData"));
 		ResetOperation(FZ_REPLY_INTERNALERROR);
 		return FZ_REPLY_ERROR;
@@ -2192,20 +2165,22 @@ int CSftpControlSocket::DeleteSend()
 	CSftpDeleteOpData *pData = static_cast<CSftpDeleteOpData *>(m_pCurOpData);
 
 	const wxString& file = pData->files.front();
-	if (file.empty())
-	{
+	if (file.empty()) {
 		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Info, _T("Empty filename"));
 		ResetOperation(FZ_REPLY_INTERNALERROR);
 		return FZ_REPLY_ERROR;
 	}
 
 	wxString filename = pData->path.FormatFilename(file);
-	if (filename.empty())
-	{
+	if (filename.empty()) {
 		LogMessage(MessageType::Error, _("Filename cannot be constructed for directory %s and filename %s"), pData->path.GetPath(), file);
+		ResetOperation(FZ_REPLY_ERROR);
 		return FZ_REPLY_ERROR;
 	}
 
+	if (!pData->m_time.IsValid())
+		pData->m_time = CDateTime::Now();
+
 	engine_.GetDirectoryCache().InvalidateFile(*m_pCurrentServer, pData->path, file);
 
 	if (!SendCommand(_T("rm ") + WildcardEscape(QuoteFilename(filename)),
