@@ -36,6 +36,10 @@
   #define EAI_ADDRFAMILY EAI_FAMILY
 #endif
 
+#ifndef AI_NUMERICSERV
+#define AI_NUMERICSERV 0
+#endif
+
 // Union for strict aliasing-safe casting between
 // the different address types
 union sockaddr_u
@@ -54,7 +58,10 @@ union sockaddr_u
 #define WAIT_EVENTCOUNT 5
 
 class CSocketThread;
-static std::list<CSocketThread*> waiting_socket_threads;
+namespace {
+static std::vector<CSocketThread*> waiting_socket_threads;
+static mutex waiting_socket_threads_mutex{false};
+};
 
 struct socket_event_type;
 typedef CEvent<socket_event_type> CInternalSocketEvent;
@@ -149,6 +156,10 @@ static int ConvertMSWErrorCode(int error)
 		return ETIMEDOUT;
 	case WSAECONNRESET:
 		return ECONNRESET;
+	case WSAEHOSTDOWN:
+		return EHOSTDOWN;
+	case WSAENETUNREACH:
+		return ENETUNREACH;
 	default:
 		return error;
 	}
@@ -193,23 +204,12 @@ public:
 	CSocketThread()
 		: wxThread(wxTHREAD_JOINABLE), m_sync(false)
 	{
-		m_pSocket = 0;
-		m_pHost = 0;
-		m_pPort = 0;
 #ifdef __WXMSW__
 		m_sync_event = WSA_INVALID_EVENT;
 #else
 		m_pipe[0] = -1;
 		m_pipe[1] = -1;
 #endif
-		m_started = false;
-		m_quit = false;
-		m_finished = false;
-
-		m_waiting = 0;
-		m_triggered = 0;
-		m_threadwait = false;
-
 		for (int i = 0; i < WAIT_EVENTCOUNT; ++i) {
 			m_triggered_errors[i] = 0;
 		}
@@ -238,34 +238,32 @@ public:
 	{
 		m_pSocket = pSocket;
 
-		delete [] m_pHost;
-		m_pHost = 0;
-
-		delete [] m_pPort;
-		m_pPort = 0;
+		m_host.clear();
+		m_port.clear();
 
 		m_waiting = 0;
 	}
 
-	int Connect()
+	int Connect(wxString const& bind)
 	{
 		wxASSERT(m_pSocket);
-		delete [] m_pHost;
-		delete [] m_pPort;
 
-		const wxWX2MBbuf buf = m_pSocket->m_host.mb_str();
-		if (!buf) {
-			m_pHost = 0;
-			m_pPort = 0;
+		auto const hostBuf = m_pSocket->m_host.mb_str();
+		auto const bindBuf = bind.mb_str();
+		if (!hostBuf || !bindBuf) {
+			m_bind.clear();
+			m_host.clear();
+			m_port.clear();
 			return EINVAL;
 		}
-		m_pHost = new char[strlen(buf) + 1];
-		strcpy(m_pHost, buf);
+		m_host = hostBuf;
+		m_bind = bindBuf;
 
 		// Connect method of CSocket ensures port is in range
-		m_pPort = new char[6];
-		sprintf(m_pPort, "%u", m_pSocket->m_port);
-		m_pPort[5] = 0;
+		char tmp[7];
+		sprintf(tmp, "%u", m_pSocket->m_port);
+		tmp[5] = 0;
+		m_port = tmp;
 
 		Start();
 
@@ -335,15 +333,15 @@ public:
 	}
 
 protected:
-	static int CreateSocketFd(addrinfo const* addr)
+	static int CreateSocketFd(addrinfo const& addr)
 	{
 		int fd;
 #if defined(SOCK_CLOEXEC) && !defined(__WXMSW__)
-		fd = socket(addr->ai_family, addr->ai_socktype | SOCK_CLOEXEC, addr->ai_protocol);
+		fd = socket(addr.ai_family, addr.ai_socktype | SOCK_CLOEXEC, addr.ai_protocol);
 		if (fd == -1 && errno == EINVAL)
 #endif
 		{
-			fd = socket(addr->ai_family, addr->ai_socktype, addr->ai_protocol);
+			fd = socket(addr.ai_family, addr.ai_socktype, addr.ai_protocol);
 		}
 
 		if (fd != -1) {
@@ -370,25 +368,29 @@ protected:
 		}
 	}
 
-	int TryConnectHost(struct addrinfo *addr, scoped_lock & l)
+	int TryConnectHost(addrinfo & addr, sockaddr_u const& bindAddr, scoped_lock & l)
 	{
 		if (m_pSocket->m_pEvtHandler) {
-			m_pSocket->m_pEvtHandler->SendEvent<CHostAddressEvent>(m_pSocket, CSocket::AddressToString(addr->ai_addr, addr->ai_addrlen));
+			m_pSocket->m_pEvtHandler->SendEvent<CHostAddressEvent>(m_pSocket, CSocket::AddressToString(addr.ai_addr, addr.ai_addrlen));
 		}
 
 		int fd = CreateSocketFd(addr);
 		if (fd == -1) {
 			if (m_pSocket->m_pEvtHandler) {
-				m_pSocket->m_pEvtHandler->SendEvent<CSocketEvent>(m_pSocket, addr->ai_next ? SocketEventType::connection_next : SocketEventType::connection, GetLastSocketError());
+				m_pSocket->m_pEvtHandler->SendEvent<CSocketEvent>(m_pSocket, addr.ai_next ? SocketEventType::connection_next : SocketEventType::connection, GetLastSocketError());
 			}
 
 			return 0;
 		}
 
+		if (bindAddr.sockaddr.sa_family != AF_UNSPEC && bindAddr.sockaddr.sa_family == addr.ai_family) {
+			(void)bind(fd, &bindAddr.sockaddr, sizeof(bindAddr));
+		}
+
 		CSocket::DoSetFlags(fd, m_pSocket->m_flags, m_pSocket->m_flags);
 		CSocket::DoSetBufferSizes(fd, m_pSocket->m_buffer_sizes[0], m_pSocket->m_buffer_sizes[1]);
 
-		int res = connect(fd, addr->ai_addr, addr->ai_addrlen);
+		int res = connect(fd, addr.ai_addr, addr.ai_addrlen);
 		if (res == -1) {
 #ifdef __WXMSW__
 			// Map to POSIX error codes
@@ -426,7 +428,7 @@ protected:
 
 		if (res) {
 			if (m_pSocket->m_pEvtHandler) {
-				m_pSocket->m_pEvtHandler->SendEvent<CSocketEvent>(m_pSocket, addr->ai_next ? SocketEventType::connection_next : SocketEventType::connection, res);
+				m_pSocket->m_pEvtHandler->SendEvent<CSocketEvent>(m_pSocket, addr.ai_next ? SocketEventType::connection_next : SocketEventType::connection, res);
 			}
 
 			CloseSocketFd(fd);
@@ -452,20 +454,33 @@ protected:
 	// Only call while locked
 	bool DoConnect(scoped_lock & l)
 	{
-		char* pHost;
-		char* pPort;
-
-		if (!m_pHost || !m_pPort) {
+		if (m_host.empty() || m_port.empty()) {
 			m_pSocket->m_state = CSocket::closed;
 			return false;
 		}
 
-		pHost = m_pHost;
-		m_pHost = 0;
-		pPort = m_pPort;
-		m_pPort = 0;
+		std::string host, port, bind;
+		std::swap(host, m_host);
+		std::swap(port, m_port);
+		std::swap(bind, m_bind);
 
-		struct addrinfo hints = {0};
+		sockaddr_u bindAddr{};
+
+		if (!bind.empty()) {
+			// Convert bind address
+			addrinfo bind_hints{};
+			bind_hints.ai_flags = AI_NUMERICHOST | AI_NUMERICSERV | AI_PASSIVE;
+			addrinfo *bindAddressList{};
+			int res = getaddrinfo(bind.empty() ? 0 : bind.c_str(), "0", &bind_hints, &bindAddressList);
+			if (!res && bindAddressList && bindAddressList->ai_addr) {
+				memcpy(&bindAddr.storage, bindAddressList->ai_addr, bindAddressList->ai_addrlen);
+			}
+			if (!res && bindAddressList) {
+				freeaddrinfo(bindAddressList);
+			}
+		}
+
+		addrinfo hints{};
 		hints.ai_family = m_pSocket->m_family;
 
 		l.unlock();
@@ -475,11 +490,8 @@ protected:
 		hints.ai_flags |= AI_IDN;
 #endif
 
-		struct addrinfo *addressList = 0;
-		int res = getaddrinfo(pHost, pPort, &hints, &addressList);
-
-		delete [] pHost;
-		delete [] pPort;
+		addrinfo *addressList{};
+		int res = getaddrinfo(host.c_str(), port.c_str(), &hints, &addressList);
 
 		l.lock();
 
@@ -495,7 +507,7 @@ protected:
 		// If m_pHost is set, Close() was called and Connect()
 		// afterwards, state is back at connecting.
 		// In either case, we need to abort this connection attempt.
-		if (m_pSocket->m_state != CSocket::connecting || m_pHost) {
+		if (m_pSocket->m_state != CSocket::connecting || !m_host.empty()) {
 			if (!res && addressList)
 				freeaddrinfo(addressList);
 			return false;
@@ -514,8 +526,8 @@ protected:
 			return false;
 		}
 
-		for (struct addrinfo *addr = addressList; addr; addr = addr->ai_next) {
-			res = TryConnectHost(addr, l);
+		for (addrinfo *addr = addressList; addr; addr = addr->ai_next) {
+			res = TryConnectHost(*addr, bindAddr, l);
 			if (res == -1) {
 				freeaddrinfo(addressList);
 				if (m_pSocket)
@@ -750,7 +762,7 @@ protected:
 	{
 		if (m_quit)
 			return false;
-		while (!m_pSocket || (!m_waiting && !m_pHost)) {
+		while (!m_pSocket || (!m_waiting && m_host.empty())) {
 			m_threadwait = true;
 			m_condition.wait(l);
 
@@ -820,10 +832,11 @@ protected:
 		return 0;
 	}
 
-	CSocket* m_pSocket;
+	CSocket* m_pSocket{};
 
-	char* m_pHost;
-	char* m_pPort;
+	std::string m_host;
+	std::string m_port;
+	std::string m_bind;
 
 #ifdef __WXMSW__
 	// We wait on this using WSAWaitForMultipleEvents
@@ -836,21 +849,19 @@ protected:
 	mutex m_sync;
 	condition m_condition;
 
-	bool m_started;
-	bool m_quit;
-	bool m_finished;
+	bool m_started{};
+	bool m_quit{};
+	bool m_finished{};
 
 	// The socket events we are waiting for
-	int m_waiting;
+	int m_waiting{};
 
 	// The triggered socket events
-	int m_triggered;
+	int m_triggered{};
 	int m_triggered_errors[WAIT_EVENTCOUNT];
 
 	// Thread waits for instructions
-	bool m_threadwait;
-
-	CCallback* m_synchronous_read_cb;
+	bool m_threadwait{};
 };
 
 CSocket::CSocket(CEventHandler* pEvtHandler)
@@ -897,11 +908,12 @@ void CSocket::DetachThread()
 			l.unlock();
 			delete m_pSocketThread;
 		}
-		else
-		{
+		else {
 			m_pSocketThread->m_quit = true;
 			m_pSocketThread->WakeupThread(l);
 			l.unlock();
+
+			scoped_lock wl(waiting_socket_threads_mutex);
 			waiting_socket_threads.push_back(m_pSocketThread);
 		}
 	}
@@ -910,7 +922,7 @@ void CSocket::DetachThread()
 	Cleanup(false);
 }
 
-int CSocket::Connect(wxString host, unsigned int port, address_family family /*=unspec*/)
+int CSocket::Connect(wxString const& host, unsigned int port, address_family family, wxString const& bind)
 {
 	if (m_state != none)
 		return EISCONN;
@@ -958,7 +970,7 @@ int CSocket::Connect(wxString host, unsigned int port, address_family family /*=
 
 	m_host = host;
 	m_port = port;
-	int res = m_pSocketThread->Connect();
+	int res = m_pSocketThread->Connect(bind);
 	if (res) {
 		m_state = none;
 		delete m_pSocketThread;
@@ -1024,7 +1036,7 @@ struct Error_table
 	const wxChar* const description;
 };
 
-static struct Error_table error_table[] =
+static Error_table const error_table[] =
 {
 	ERRORDECL(EACCES, TRANSLATE_T("Permission denied"))
 	ERRORDECL(EADDRINUSE, TRANSLATE_T("Local address in use"))
@@ -1056,9 +1068,10 @@ static struct Error_table error_table[] =
 	ERRORDECL(ECONNABORTED, TRANSLATE_T("Connection aborted"))
 	ERRORDECL(ECONNRESET, TRANSLATE_T("Connection reset by peer"))
 	ERRORDECL(EPIPE, TRANSLATE_T("Local endpoint has been closed"))
+	ERRORDECL(EHOSTDOWN, TRANSLATE_T("Host is down"))
 
 	// Getaddrinfo related
-#ifndef __WXMSW__
+#ifdef EAI_ADDRFAMILY
 	ERRORDECL(EAI_ADDRFAMILY, TRANSLATE_T("Network host does not have any network addresses in the requested address family"))
 #endif
 	ERRORDECL(EAI_AGAIN, TRANSLATE_T("Temporary failure in name resolution"))
@@ -1081,7 +1094,7 @@ static struct Error_table error_table[] =
 #endif
 	ERRORDECL(EAI_SERVICE, TRANSLATE_T("The servname parameter is not supported for ai_socktype"))
 	ERRORDECL(EAI_SOCKTYPE, TRANSLATE_T("The ai_socktype member is not supported"))
-#ifndef __WXMSW__
+#ifdef EAI_SYSTEM
 	ERRORDECL(EAI_SYSTEM, TRANSLATE_T("Other system error"))
 #endif
 
@@ -1130,11 +1143,9 @@ int CSocket::Close()
 		int fd = m_fd;
 		m_fd = -1;
 
-		delete [] m_pSocketThread->m_pHost;
-		m_pSocketThread->m_pHost = 0;
-		delete [] m_pSocketThread->m_pPort;
-		m_pSocketThread->m_pPort = 0;
-		if (!m_pSocketThread->m_threadwait)
+		m_pSocketThread->m_host.clear();
+		m_pSocketThread->m_port.clear();
+
 			m_pSocketThread->WakeupThread(l);
 
 		CSocketThread::CloseSocketFd(fd);
@@ -1177,26 +1188,24 @@ CSocket::SocketState CSocket::GetState()
 	return state;
 }
 
-bool CSocket::Cleanup(bool force)
+void CSocket::Cleanup(bool force)
 {
+	scoped_lock wl(waiting_socket_threads_mutex);
 	auto iter = waiting_socket_threads.begin();
-	while (iter != waiting_socket_threads.end()) {
-		auto current = iter++;
-		CSocketThread* pThread = *current;
+	for (; iter != waiting_socket_threads.end(); ++iter) {
+		CSocketThread *const pThread = *iter;
 
-		{
+		if (!force) {
 			scoped_lock l(pThread->m_sync);
-			if (!force && !pThread->m_finished) {
-				continue;
+			if (!pThread->m_finished) {
+				break;
 			}
 		}
 
 		pThread->Wait(wxTHREAD_WAIT_BLOCK);
 		delete pThread;
-		waiting_socket_threads.erase(current);
 	}
-
-	return false;
+	waiting_socket_threads.erase(waiting_socket_threads.begin(), iter);
 }
 
 int CSocket::Read(void* buffer, unsigned int size, int& error)
@@ -1416,7 +1425,7 @@ int CSocket::Listen(address_family family, int port)
 		}
 
 		for (struct addrinfo* addr = addressList; addr; addr = addr->ai_next) {
-			m_fd = CSocketThread::CreateSocketFd(addr);
+			m_fd = CSocketThread::CreateSocketFd(*addr);
 			res = GetLastSocketError();
 
 			if (m_fd == -1)
@@ -1640,4 +1649,3 @@ wxString CSocket::GetPeerHost() const
 {
 	return m_host;
 }
-
