@@ -8,7 +8,7 @@
 class COptions;
 
 enum {
-	changed_options_size = 128
+	changed_options_size = 64*3
 };
 
 typedef std::bitset<changed_options_size> changed_options_t;
@@ -30,7 +30,10 @@ protected:
 private:
 	changed_options_t m_handled_options;
 
+	// Very important: Never ever call this if there's OnOptionsChanged on the stack.
 	static void DoNotify(changed_options_t const& options);
+	static std::size_t notify_index_;
+
 	static void UnregisterAll();
 
 	static std::vector<COptionChangeEventHandler*> m_handlers;
