@@ -248,7 +248,6 @@ CRemoteTreeView::CRemoteTreeView(wxWindow* parent, wxWindowID id, CState* pState
 #endif
 
 	pState->RegisterHandler(this, STATECHANGE_REMOTE_DIR);
-	pState->RegisterHandler(this, STATECHANGE_REMOTE_DIR_MODIFIED);
 	pState->RegisterHandler(this, STATECHANGE_APPLYFILTER);
 
 	CreateImageList();
@@ -272,12 +271,10 @@ CRemoteTreeView::~CRemoteTreeView()
 	delete m_pImageList;
 }
 
-void CRemoteTreeView::OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString&, const void*)
+void CRemoteTreeView::OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString&, const void* v)
 {
 	if (notification == STATECHANGE_REMOTE_DIR)
-		SetDirectoryListing(pState->GetRemoteDir(), false);
-	else if (notification == STATECHANGE_REMOTE_DIR_MODIFIED)
-		SetDirectoryListing(pState->GetRemoteDir(), true);
+		SetDirectoryListing(pState->GetRemoteDir(), v ? *reinterpret_cast<bool const*>(v) : false);
 	else if (notification == STATECHANGE_APPLYFILTER)
 		ApplyFilters(false);
 }
@@ -522,8 +519,7 @@ void CRemoteTreeView::DisplayItem(wxTreeItemId parent, const CDirectoryListing&
 	const wxString path = listing.path.GetPath();
 
 	CFilterDialog filter;
-	for (unsigned int i = 0; i < listing.GetCount(); i++)
-	{
+	for (unsigned int i = 0; i < listing.GetCount(); ++i) {
 		if (!listing[i].is_dir())
 			continue;
 
@@ -536,16 +532,14 @@ void CRemoteTreeView::DisplayItem(wxTreeItemId parent, const CDirectoryListing&
 
 		CDirectoryListing subListing;
 
-		if (m_pState->m_pEngine->CacheLookup(subdir, subListing) == FZ_REPLY_OK)
-		{
+		if (m_pState->m_pEngine->CacheLookup(subdir, subListing) == FZ_REPLY_OK) {
 			wxTreeItemId child = AppendItem(parent, name, 0, 2, 0);
 			SetItemImages(child, false);
 
 			if (HasSubdirs(subListing, filter))
 				AppendItem(child, _T(""), -1, -1);
 		}
-		else
-		{
+		else {
 			wxTreeItemId child = AppendItem(parent, name, 1, 3, 0);
 			SetItemImages(child, true);
 		}
@@ -559,19 +553,17 @@ void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing&
 
 	wxTreeItemIdValue cookie;
 	wxTreeItemId child = GetFirstChild(parent, cookie);
-	if (!child || GetItemText(child).empty())
-	{
+	if (!child || GetItemText(child).empty()) {
 		DisplayItem(parent, listing);
 		return;
 	}
 
 	CFilterManager filter;
 
-	const wxString path = listing.path.GetPath();
+	wxString const path = listing.path.GetPath();
 
 	wxArrayString dirs;
-	for (unsigned int i = 0; i < listing.GetCount(); i++)
-	{
+	for (unsigned int i = 0; i < listing.GetCount(); ++i) {
 		if (!listing[i].is_dir())
 			continue;
 
@@ -589,12 +581,11 @@ void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing&
 		int cmp = sortFunc(GetItemText(child), *iter);
 
 		if (!cmp) {
-			CServerPath path = listing.path;
-			path.AddSegment(*iter);
+			CServerPath childPath = listing.path;
+			childPath.AddSegment(*iter);
 
 			CDirectoryListing subListing;
-			if (m_pState->m_pEngine->CacheLookup(path, subListing) == FZ_REPLY_OK)
-			{
+			if (m_pState->m_pEngine->CacheLookup(childPath, subListing) == FZ_REPLY_OK) {
 				if (!GetLastChild(child) && HasSubdirs(subListing, filter))
 					AppendItem(child, _T(""), -1, -1);
 				SetItemImages(child, false);
@@ -617,21 +608,25 @@ void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing&
 		}
 		else if (cmp < 0) {
 			// New directory
-			CServerPath path = listing.path;
-			path.AddSegment(*iter);
+			CServerPath childPath = listing.path;
+			childPath.AddSegment(*iter);
 
 			CDirectoryListing subListing;
-			if (m_pState->m_pEngine->CacheLookup(path, subListing) == FZ_REPLY_OK) {
-				wxTreeItemId child = AppendItem(parent, *iter, 0, 2, 0);
-				SetItemImages(child, false);
+			if (m_pState->m_pEngine->CacheLookup(childPath, subListing) == FZ_REPLY_OK) {
+				wxTreeItemId childItem = AppendItem(parent, *iter, 0, 2, 0);
+				if (childItem) {
+					SetItemImages(childItem, false);
 
-				if (HasSubdirs(subListing, filter))
-					AppendItem(child, _T(""), -1, -1);
+					if (HasSubdirs(subListing, filter)) {
+						AppendItem(childItem, _T(""), -1, -1);
+					}
+				}
 			}
 			else {
-				wxTreeItemId child = AppendItem(parent, *iter, 1, 3, 0);
-				if (child)
-					SetItemImages(child, true);
+				wxTreeItemId childItem = AppendItem(parent, *iter, 1, 3, 0);
+				if (childItem) {
+					SetItemImages(childItem, true);
+				}
 			}
 
 			++iter;
@@ -649,20 +644,25 @@ void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing&
 		child = prev;
 	}
 	while (iter != dirs.rend()) {
-		CServerPath path = listing.path;
-		path.AddSegment(*iter);
+		CServerPath childPath = listing.path;
+		childPath.AddSegment(*iter);
 
 		CDirectoryListing subListing;
-		if (m_pState->m_pEngine->CacheLookup(path, subListing) == FZ_REPLY_OK) {
-			wxTreeItemId child = AppendItem(parent, *iter, 0, 2, 0);
-			SetItemImages(child, false);
+		if (m_pState->m_pEngine->CacheLookup(childPath, subListing) == FZ_REPLY_OK) {
+			wxTreeItemId childItem = AppendItem(parent, *iter, 0, 2, 0);
+			if (childItem) {
+				SetItemImages(childItem, false);
 
-			if (HasSubdirs(subListing, filter))
-				AppendItem(child, _T(""), -1, -1);
+				if (HasSubdirs(subListing, filter)) {
+					AppendItem(childItem, _T(""), -1, -1);
+				}
+			}
 		}
 		else {
-			wxTreeItemId child = AppendItem(parent, *iter, 1, 3, 0);
-			SetItemImages(child, true);
+			wxTreeItemId childItem = AppendItem(parent, *iter, 1, 3, 0);
+			if (childItem) {
+				SetItemImages(childItem, true);
+			}
 		}
 
 		++iter;
@@ -721,8 +721,7 @@ void CRemoteTreeView::OnSelectionChanged(wxTreeEvent& event)
 	if (m_busy)
 		return;
 
-	if (!m_pState->IsRemoteIdle())
-	{
+	if (!m_pState->IsRemoteIdle(true)) {
 		wxBell();
 		return;
 	}
@@ -774,8 +773,7 @@ CServerPath CRemoteTreeView::GetPathFromItem(const wxTreeItemId& item) const
 void CRemoteTreeView::OnBeginDrag(wxTreeEvent& event)
 {
 	// Drag could result in recursive operation, don't allow at this point
-	if (!m_pState->IsRemoteIdle())
-	{
+	if (!m_pState->IsRemoteIdle()) {
 		wxBell();
 		return;
 	}
@@ -784,7 +782,7 @@ void CRemoteTreeView::OnBeginDrag(wxTreeEvent& event)
 	if (!item)
 		return;
 
-	const CServerPath& path = GetPathFromItem(item);
+	CServerPath path = GetPathFromItem(item);
 	if (path.empty() || !path.HasParent())
 		return;
 
@@ -795,9 +793,11 @@ void CRemoteTreeView::OnBeginDrag(wxTreeEvent& event)
 
 	wxDataObjectComposite object;
 
-	const CServer* const pServer = m_pState->GetServer();
+	CServer const* pServer = m_pState->GetServer();
 	if (!pServer)
 		return;
+	CServer const server = *pServer;
+
 
 	CRemoteDataObject *pRemoteDataObject = new CRemoteDataObject(*pServer, parent);
 	pRemoteDataObject->AddFile(lastSegment, true, -1, false);
@@ -807,9 +807,8 @@ void CRemoteTreeView::OnBeginDrag(wxTreeEvent& event)
 	object.Add(pRemoteDataObject, true);
 
 #if FZ3_USESHELLEXT
-	CShellExtensionInterface* ext = CShellExtensionInterface::CreateInitialized();
-	if (ext)
-	{
+	std::unique_ptr<CShellExtensionInterface> ext = CShellExtensionInterface::CreateInitialized();
+	if (ext) {
 		const wxString& file = ext->GetDragDirectory();
 
 		wxASSERT(!file.empty());
@@ -833,46 +832,28 @@ void CRemoteTreeView::OnBeginDrag(wxTreeEvent& event)
 
 	pDragDropManager->Release();
 
-	if (res != wxDragCopy)
-	{
-#if FZ3_USESHELLEXT
-		delete ext;
-		ext = 0;
-#endif
+	if (res != wxDragCopy) {
 		return;
 	}
 
 #if FZ3_USESHELLEXT
-	if (ext)
-	{
-		if (!pRemoteDataObject->DidSendData())
-		{
-			const CServer* const pServer = m_pState->GetServer();
-			if (!pServer || !m_pState->IsRemoteIdle())
-			{
+	if (ext) {
+		if (!pRemoteDataObject->DidSendData()) {
+			pServer = m_pState->GetServer();
+			if (!pServer || !m_pState->IsRemoteIdle() || *pServer != server) {
 				wxBell();
-				delete ext;
-				ext = 0;
 				return;
 			}
 
 			CLocalPath target(ext->GetTarget());
-			if (target.empty())
-			{
-				delete ext;
-				ext = 0;
+			if (target.empty()) {
+				ext.reset(); // Release extension before the modal message box
 				wxMessageBoxEx(_("Could not determine the target of the Drag&Drop operation.\nEither the shell extension is not installed properly or you didn't drop the files into an Explorer window."));
 				return;
 			}
 
 			m_pState->DownloadDroppedFiles(pRemoteDataObject, target);
-
-			delete ext;
-			ext = 0;
-			return;
 		}
-		delete ext;
-		ext = 0;
 	}
 #endif
 }
@@ -885,8 +866,7 @@ void CRemoteTreeView::OnContextMenu(wxTreeEvent& event)
 		return;
 
 	const CServerPath& path = m_contextMenuItem ? GetPathFromItem(m_contextMenuItem) : CServerPath();
-	if (!m_pState->IsRemoteIdle() || path.empty())
-	{
+	if (!m_pState->IsRemoteIdle() || path.empty()) {
 		pMenu->Enable(XRCID("ID_DOWNLOAD"), false);
 		pMenu->Enable(XRCID("ID_ADDTOQUEUE"), false);
 		pMenu->Enable(XRCID("ID_MKDIR"), false);
@@ -899,8 +879,7 @@ void CRemoteTreeView::OnContextMenu(wxTreeEvent& event)
 	else if (!path.HasParent())
 		pMenu->Enable(XRCID("ID_RENAME"), false);
 
-	if (!m_pState->GetLocalDir().IsWriteable())
-	{
+	if (!m_pState->GetLocalDir().IsWriteable()) {
 		pMenu->Enable(XRCID("ID_DOWNLOAD"), false);
 		pMenu->Enable(XRCID("ID_ADDTOQUEUE"), false);
 	}
@@ -931,17 +910,14 @@ void CRemoteTreeView::OnMenuChmod(wxCommandEvent&)
 	bool cached = false;
 
 	// Obviously item needs to have a parent directory...
-	if (hasParent)
-	{
+	if (hasParent) {
 		const CServerPath& parentPath = path.GetParent();
 		CDirectoryListing listing;
 
 		// ... and it needs to be cached
 		cached = m_pState->m_pEngine->CacheLookup(parentPath, listing) == FZ_REPLY_OK;
-		if (cached)
-		{
-			for (unsigned int i = 0; i < listing.GetCount(); i++)
-			{
+		if (cached) {
+			for (unsigned int i = 0; i < listing.GetCount(); ++i) {
 				if (listing[i].name != name)
 					continue;
 
@@ -950,23 +926,20 @@ void CRemoteTreeView::OnMenuChmod(wxCommandEvent&)
 		}
 	}
 
-	if (!pChmodDlg->Create(this, 0, 1, name, permissions))
-	{
+	if (!pChmodDlg->Create(this, 0, 1, name, permissions)) {
 		pChmodDlg->Destroy();
 		pChmodDlg = 0;
 		return;
 	}
 
-	if (pChmodDlg->ShowModal() != wxID_OK)
-	{
+	if (pChmodDlg->ShowModal() != wxID_OK) {
 		pChmodDlg->Destroy();
 		pChmodDlg = 0;
 		return;
 	}
 
 	// State may have changed while chmod dialog was shown
-	if (!m_contextMenuItem || !m_pState->IsRemoteConnected() || !m_pState->IsRemoteIdle())
-	{
+	if (!m_contextMenuItem || !m_pState->IsRemoteConnected() || !m_pState->IsRemoteIdle()) {
 		pChmodDlg->Destroy();
 		pChmodDlg = 0;
 		return;
@@ -976,11 +949,9 @@ void CRemoteTreeView::OnMenuChmod(wxCommandEvent&)
 
 	CRecursiveOperation* pRecursiveOperation = m_pState->GetRecursiveOperationHandler();
 
-	if (cached) // Implies hasParent
-	{
+	if (cached) { // Implies hasParent
 		// Change directory permissions
-		if (!applyType || applyType == 2)
-		{
+		if (!applyType || applyType == 2) {
 			wxString newPerms = pChmodDlg->GetPermissions(permissions, true);
 
 			m_pState->m_pCommandQueue->ProcessCommand(new CChmodCommand(path.GetParent(), name, newPerms));
@@ -990,16 +961,14 @@ void CRemoteTreeView::OnMenuChmod(wxCommandEvent&)
 			// Start recursion
 			pRecursiveOperation->AddDirectoryToVisit(path, _T(""), CLocalPath());
 	}
-	else
-	{
+	else {
 		if (hasParent)
 			pRecursiveOperation->AddDirectoryToVisitRestricted(path.GetParent(), name, pChmodDlg->Recursive());
 		else
 			pRecursiveOperation->AddDirectoryToVisitRestricted(path, _T(""), pChmodDlg->Recursive());
 	}
 
-	if (!cached || pChmodDlg->Recursive())
-	{
+	if (!cached || pChmodDlg->Recursive()) {
 		pRecursiveOperation->SetChmodDialog(pChmodDlg);
 
 		CServerPath currentPath;
@@ -1009,12 +978,10 @@ void CRemoteTreeView::OnMenuChmod(wxCommandEvent&)
 		CFilterManager filter;
 		pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_chmod, hasParent ? path.GetParent() : path, filter.GetActiveFilters(false), !cached, currentPath);
 	}
-	else
-	{
+	else {
 		pChmodDlg->Destroy();
 		const wxTreeItemId selected = GetSelection();
-		if (selected)
-		{
+		if (selected) {
 			CServerPath currentPath = GetPathFromItem(selected);
 			m_pState->ChangeRemoteDir(currentPath);
 		}
@@ -1024,8 +991,7 @@ void CRemoteTreeView::OnMenuChmod(wxCommandEvent&)
 void CRemoteTreeView::OnMenuDownload(wxCommandEvent& event)
 {
 	CLocalPath localDir = m_pState->GetLocalDir();
-	if (!localDir.IsWriteable())
-	{
+	if (!localDir.IsWriteable()) {
 		wxBell();
 		return;
 	}
@@ -1120,21 +1086,18 @@ void CRemoteTreeView::OnMenuRename(wxCommandEvent&)
 
 void CRemoteTreeView::OnBeginLabelEdit(wxTreeEvent& event)
 {
-	if (!m_pState->IsRemoteIdle())
-	{
+	if (!m_pState->IsRemoteIdle()) {
 		event.Veto();
 		return;
 	}
 
 	const CServerPath& path = GetPathFromItem(event.GetItem());
-	if (path.empty())
-	{
+	if (path.empty()) {
 		event.Veto();
 		return;
 	}
 
-	if (!path.HasParent())
-	{
+	if (!path.HasParent()) {
 		event.Veto();
 		return;
 	}
@@ -1142,21 +1105,18 @@ void CRemoteTreeView::OnBeginLabelEdit(wxTreeEvent& event)
 
 void CRemoteTreeView::OnEndLabelEdit(wxTreeEvent& event)
 {
-	if (event.IsEditCancelled())
-	{
+	if (event.IsEditCancelled()) {
 		event.Veto();
 		return;
 	}
 
-	if (!m_pState->IsRemoteIdle())
-	{
+	if (!m_pState->IsRemoteIdle()) {
 		event.Veto();
 		return;
 	}
 
 	CItemData* const pData = (CItemData*)GetItemData(event.GetItem());
-	if (pData)
-	{
+	if (pData) {
 		event.Veto();
 		return;
 	}
@@ -1166,8 +1126,7 @@ void CRemoteTreeView::OnEndLabelEdit(wxTreeEvent& event)
 
 	const wxString& oldName = GetItemText(event.GetItem());
 	const wxString& newName = event.GetLabel();
-	if (oldName == newName)
-	{
+	if (oldName == newName) {
 		event.Veto();
 		return;
 	}
@@ -1182,15 +1141,12 @@ void CRemoteTreeView::OnEndLabelEdit(wxTreeEvent& event)
 	if (currentPath.empty())
 		return;
 
-	if (currentPath == old_path || currentPath.IsSubdirOf(old_path, false))
-	{
+	if (currentPath == old_path || currentPath.IsSubdirOf(old_path, false)) {
 		// Previously selected path was below renamed one, list the new one
 
 		std::list<wxString> subdirs;
-		while (currentPath != old_path)
-		{
-			if (!currentPath.HasParent())
-			{
+		while (currentPath != old_path) {
+			if (!currentPath.HasParent()) {
 				// Abort just in case
 				return;
 			}
@@ -1262,8 +1218,7 @@ CServerPath CRemoteTreeView::MenuMkdir()
 	// replace it with "New directory" later. This way we get the exact position of
 	// "New directory" and can preselect it in the dialog.
 	wxString tmpName = _T("25CF809E56B343b5A12D1F0466E3B37A49A9087FDCF8412AA9AF8D1E849D01CF");
-	if (newPath.AddSegment(tmpName))
-	{
+	if (newPath.AddSegment(tmpName)) {
 		wxString pathName = newPath.GetPath();
 		int pos = pathName.Find(tmpName);
 		wxASSERT(pos != -1);
@@ -1277,8 +1232,7 @@ CServerPath CRemoteTreeView::MenuMkdir()
 		return CServerPath();
 
 	newPath = path;
-	if (!newPath.ChangePath(dlg.GetValue()))
-	{
+	if (!newPath.ChangePath(dlg.GetValue())) {
 		wxBell();
 		return CServerPath();
 	}
